//Definitions for Lava Code
//Created by Mcshay
//Verison 1.0

#ifndef LSTRUCTS_H
#define LSTRUCTS_H

#include <vector>

using namespace std;

typedef void* VPointer;
typedef int NullMessageBody;
extern NullMessageBody NullParam;

///Volcano Message Chart:
///--vmNoMessage---------
/*BodyA: NULL
  BodyB: NULL
  BodyC: NULL */
///--vmNewNode-----------
/*BodyA: LavaNode* EmptyNode
  BodyB: LavaNode* ExistingNode
  BodyC: Alloc(int RadiusForEmptyNode) */
///--vmNewPath-----------
/*BodyA: LavaNode* NodeA
  BodyB: LavaNode* NodeB
  BodyC: NULL */
///--vmExpandNode--------
/*BodyA: LavaNode* Node
  BodyB: Alloc(int NumberOfTilesToAddToExistingRadius)
  BodyC: NULL */
///--vmExpandPath--------
/*BodyA: LavaNode* NodeA
  BodyB: LavaNode* NodeB
  BodyC: NULL */
///--vmToggleActive------
/*BodyA: Alloc(bool DoActivate)
  BodyB: NULL
  BodyC: NULL */
///--vmToggleAutomated---
/*BodyA: Alloc(bool DoAutomate)
  BodyB: NULL
  BodyC: NULL */
///--vmSetTimer----------
/*BodyA: Alloc(int MinTime)
  BodyB: Alloc(int MaxTime) <- optional(-1) for a fixed timer of MinTime
  BodyC: NULL */
///--vmSetMaxNodes-------
/*BodyA: Alloc(int NumNewMaxNodes) <- set to a number less than 0 for unlimited nodes
  BodyB: NULL
  BodyC: NULL */

enum VMessage {
    vmNoMessage = -1,
    vmNewNode = 0,
    vmNewPath = 1,
    vmExpandNode = 2,
    vmExpandPath = 3,
    vmToggleActive = 4,
    vmToggleAutomated = 5,
    vmSetTimer = 6,
    vmSetMaxNodes = 7,
};

enum NodeType {
    ntEmptyNode = 0,
    ntPartialNode = 1,
    ntFullNode = 2,
};

enum ConnectionType {
    ctNoConnection = 0,
    ctOutdated = 1,
    ctUpdated = 2,
};

//Need to tell NeighborData LavaNode exists
class LavaNode;

class NeighborData {
public:
    NeighborData();
    NeighborData(LavaNode* node, ConnectionType ctype, bool nstate);
    LavaNode* Node;
    ConnectionType CType;
    bool NeighborState; //true if considered neighbors
};

class BasicNode {
public:
    BasicNode();
    BasicNode(LOCATION loc, int maxradius, int numNeighbors, ...);
    ~BasicNode();
    LOCATION Loc;
    int MaxRadius;
    vector<int> BasicNodes;//NodeIndexes
};

class LavaNode {
public:
    LavaNode();
    LavaNode(LOCATION loc, int maxradius);
    LavaNode(BasicNode* BNode);
    LOCATION Loc;
    int Radius; //Does not include the center
    int MaxRadius;
    NodeType NType;
    vector<NeighborData> NeighborNodes;
};

class Path {
public:
    Path();
    Path(int numNodes, ...);
    ~Path();
    int* NodeIndexes;
    int NumNodeIndexes;
};

struct VolcanoTree {
    bool IsActive;
    int MessageQueueIndex;
    int NumNodes;
    int MaxNodes;
    bool IgnoreNodeLimit;
    bool IsAutomated;
    int MinTime;
    int MaxTime;
    bool IsRange;
    int TickCounter;
    int TriggerTick;
    LavaNode* VolcanoNode;
    vector<LavaNode*> FlowNodes;
};

class MessageCompare;

#endif
