//SmartLava.h
//Created by Mcshay
//Verison 1.0

#ifndef SMARTLAVA_H
#define SMARTLAVA_H

#include <windows.h>
#include <vector>
#include <queue>
#include <algorithm>
#include <cstdarg>
#include "Outpost2DLL.h"
#include "LStructs.h"

//----------------------------Requried functions--------------------------------
//Sets essential variables, MUST be called at the start of InitProc
void SmartLavaInit(LavaNode* LavaNodeArray, int NumNodes, VolcanoTree* VolcanoArray, int NumVolcanoes);
//Must be placed inside of AIProc
void LavaAIProc();
//Must be placed in DllMain, after this line: else if(fdwReason == DLL_PROCESS_DETACH)
void SmartLavaCleanUp();

//--------------------------Map Handling Functions------------------------------
//(Re)sets a list of all cell types the engine considers lava passable
void SetLavaCells(int NumCells, ...);
//(Re)sets a list of all cell types the engine considers impassible(by units)
void SetImpassibleCells(int NumCells, ...);
//Adds an entry to one of the above lists
void AddCellType(bool IsLava, int CellType);
//Changes a cell type on the engine's copy of the cell type layer
void ChangeMapCellType(LOCATION Loc, int CellType);
//returns true if the celltype at the location is considered lava passable
bool GetMatchLavaCell(LOCATION Loc);
//returns true if the celltype at the location is considered impassible
bool GetIsImpassible(LOCATION Loc);
//returns true if the location is on the map
bool IsOnMap(LOCATION Loc);
//returns Loc if it's on the map, or else it returns the closest Location on the map
LOCATION ClipLocToMap(LOCATION Loc);

//Functions to start the lava flow animation on the side of a volcano
void FlowSW(LOCATION TopLeftMostTile);
void FlowS(LOCATION TopLeftMostTile);
void FlowSE(LOCATION TopLeftMostTile);
//Functions to stop the lava flow animation on the side of a volcano
void FreezeSW(LOCATION TopLeftMostTile);
void FreezeS(LOCATION TopLeftMostTile);
void FreezeSE(LOCATION TopLeftMostTile);

//------------------Allocation & Message Functions------------------------------
//returns a pointer to a block of memory equal to the given variable
int* Alloc(int IntToAlloc);
bool* Alloc(bool BoolToAlloc);
//Creates a message for the specified volcano
bool WriteMessage(VMessage VMdesc, int ProcessTick, int VolcanoIndex, VPointer BodyA, VPointer BodyB, VPointer BodyC);

//----------------------Class Handling Functions--------------------------------
//Loads a volcano at the given node, must be called before the volcano can be used
void LoadVolcano(VolcanoTree* VTree, int MaxNodes, LavaNode* VNode, int RadiusToLoad, bool Automate, int minTime, int maxTime);
//List of LavaNode* vars & bools to be made neighbors, you don't need to call this function if you use BasicNode classes
void PopulateNeighbors(LavaNode* Node, int NumNeighbors, ...);
//returns a pointer to HomeNode's NeighborData class that references NeighborNode, returns NULL if none exists
NeighborData* GetNeighborData(LavaNode* HomeNode, LavaNode* NeighborNode);
//Randomly prevents one of the Paths from the ArrayOfPaths from being covered with lava
void MarkPath(Path ArrayOfPaths[], int ArraySize);
//returns true if NeighborNode has CheckNode as one of it's neighbors
bool IsNeighbor(LavaNode* NeighborNode, LavaNode* CheckNode);

//--------------------------Testing Functions-----------------------------------
//Draws out neighbor connections on the map, OnlyDrawNodes will only place markers for each node's center
void DrawAllConnections(bool OnlyDrawNodes);
//Marks all nodes lava possible at their max radius
void FloodAllNodes();
//if TurnOn is true, any error messages will be shown, if TurnOn is false, they won't be shown anymore
void SetDoShowErrors(bool TurnOn);

#endif
